unit Load_Save;

{
    Vector graphical editor.
    Version: 1.
    Written on FreePascal (https://freepascal.org/).
    Copyright (C) 2023  Warfalomay

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
}

{$mode ObjFPC}{$H+}

interface
//Добавить из векторного файла
Procedure Add_vect(f_name:string);
//Загрузить веккторный файл
Procedure Load_vect(f_name_op:string);
//Сохраниить векторный файл
Procedure Save_vect(f_name:string);
//Сохранить скрипт VB-Nanocad
Procedure SaveScript();
//Сохранить растровый фрагмент в буфер
Procedure SaveToBuf();


implementation
Uses
Graphics,Unit1,Classes, SysUtils, StrUtils, My_modify, Dialogs,
Clipbrd, math;

Procedure SaveToBuf();

var
   vremPic, PicX : TBitmap;
   count : integer;
   xMin : integer = 100000;
   xMax : integer = -1;
   yMin : integer = 100000;
   yMax : integer = -1;

   LeftBorder, RightBorder,
   TopBorder, BottomBorder : integer;

begin
   vremPic := TBitmap.Create;
   PicX := TBitmap.Create;
   //
   vremPic.Width := 1000;
   vremPic.Height := 1000;

   //По линиям
   for count := 0 to High(multy_selected_line) do
   begin
       xMin := min(lines[multy_selected_line[count]].x1+Screen_x,xMin);
       xMin := min(lines[multy_selected_line[count]].x2+Screen_x,xMin);
       yMin := min(lines[multy_selected_line[count]].y1+Screen_y,yMin);
       yMin := min(lines[multy_selected_line[count]].y2+Screen_y,yMin);

       xMax := max(lines[multy_selected_line[count]].x1+Screen_x,xMax);
       xMax := max(lines[multy_selected_line[count]].x2+Screen_x,xMax);
       yMax := max(lines[multy_selected_line[count]].y1+Screen_y,yMax);
       yMax := max(lines[multy_selected_line[count]].y2+Screen_y,yMax);
   end;
   //По окружностям
   for count := 0 to High(multy_selected_circle) do
   with circles[multy_selected_circle[count]] do
   begin
       xMin := min(x-r+Screen_x,xMin);
       xMax := max(x+r+Screen_x,xMax);
       yMin := min(y-r+Screen_y,yMin);
       yMax := max(y+r+Screen_y,yMax);
   end;
   //По точкам
   for count := 0 to High(multy_selected_point) do
   with points[multy_selected_point[count]] do
   begin
       xMin := min(x+Screen_x,xMin);
       xMax := max(x+Screen_x,xMax);
       yMin := min(y+Screen_y,yMin);
       yMax := max(y+Screen_y,yMax);
   end;
   //По дугам
   for count := 0 to High(multy_selected_arcs) do
   with arcs[multy_selected_arcs[count]] do
   begin
       xMin := min(l_x+Screen_x,xMin);
       xMax := max(r_x+Screen_x,xMax);
       yMin := min(t_y+Screen_y,yMin);
       yMax := max(b_y+Screen_y,yMax);
   end;
   //По текстам
   for count := 0 to High(multy_selected_text) do
   with nadp[multy_selected_text[count]] do
   begin
       xMin := min(x+Screen_x,xMin);
       xMax := max(x+h*Length(Utf8ToAnsi(s))+Screen_x,xMax);
       yMin := min(y+Screen_y,yMin);
       yMax := max(y+h+Screen_y,yMax);
   end;
   //По стрелкам
   for count := 0 to High(multy_selected_arrow) do
   With arrow[multy_selected_arrow[count]] do
   begin
       xMin := min(x-30+Screen_x,xMin);
       xMax := max(x+30+Screen_x,xMax);
       yMin := min(y-30+Screen_y,yMin);
       yMax := max(y+30+Screen_y,yMax);
   end;

   //showmessage(xMin.ToString+ '/' + yMin.ToString + '-/-' +
   //            xMax.ToString+ '/' + yMax.ToString);

   LeftBorder := xMin + Screen_x - 5;
   RightBorder := xMax + Screen_x + 5;
   TopBorder := yMin + Screen_y - 5;
   BottomBorder := yMax + Screen_y + 5;

   //PicX.Width := abs(xMax-xMin)+10;
   //PicX.Height := abs(YMax-yMin)+10;

   vremPic.Canvas.Brush.Color := clWhite;
   vremPic.Canvas.Rectangle(0,0,vremPic.Width,vremPic.Height);
   //
   //Отрисовка линий во фрагмент
   for count := 0 to High(multy_selected_line) do
   begin
       vremPic.Canvas.Pen.Color := lines[multy_selected_line[count]].col;
       vremPic.Canvas.Pen.Style := lines[multy_selected_line[count]].style;
       vremPic.Canvas.Pen.Width := lines[multy_selected_line[count]].Width;
       With lines[multy_selected_line[count]] do
            vremPic.Canvas.Line(x1+Screen_x, y1 + Screen_y, x2+Screen_x, y2+Screen_y);
   end;
   //Отрисовка окружностей во фрагмент
   for count := 0 to High(multy_selected_circle) do
   begin
       vremPic.Canvas.Pen.Color := circles[multy_selected_circle[count]].col;
       vremPic.Canvas.Pen.Style := circles[multy_selected_circle[count]].style;
       vremPic.Canvas.Pen.Width := circles[multy_selected_circle[count]].Width;
       vremPic.Canvas.Brush.Style := bsClear;
       With circles[multy_selected_circle[count]] do
            vremPic.Canvas.Ellipse(x-r+Screen_x, y-r+Screen_y, x+r+Screen_x, y+r+Screen_y);
   end;
   //Отрисовка точек во фрагмент
   for count := 0 to High(multy_selected_point) do
   begin
       vremPic.Canvas.Ellipse(points[multy_selected_point[count]].x-3+Screen_x,
                              points[multy_selected_point[count]].y-3+Screen_y,
                              points[multy_selected_point[count]].x+3+Screen_x,
                              points[multy_selected_point[count]].y+3+Screen_y);
   end;
   //Отрисовка текстов во фрагмент
   for count := 0 to High(multy_selected_text) do
   begin
       vremPic.Canvas.Font.Color := nadp[count].col;
       vremPic.Canvas.Font.Size := nadp[count].h;
       vremPic.Canvas.TextOut(nadp[multy_selected_text[count]].x+Screen_x,
                              nadp[multy_selected_text[count]].y+Screen_y,
                              nadp[multy_selected_text[count]].s);
   end;
   //Отрисовка стрелок во фрагмент
   for count := 0 to High(multy_selected_arrow) do
   begin
       vremPic.Canvas.Pen.Color := arrow[multy_selected_arrow[count]].col;

       With arrow[multy_selected_arrow[count]] do
       begin
           If arrow[multy_selected_arrow[count]].t='left' then
               begin
                   vremPic.Canvas.Line(x+Screen_x,y+Screen_y,x+30+Screen_x,y+Screen_y);
                   vremPic.Canvas.Line(x+Screen_x,y+Screen_y,x+10+Screen_x,y+5+Screen_y);
                   vremPic.Canvas.Line(x+Screen_x,y+Screen_y,x+10+Screen_x,y-5+Screen_y);
               end;
           If arrow[multy_selected_arrow[count]].t='right' then
               begin
                   vremPic.Canvas.Line(x+Screen_x,y+Screen_y,x-30+Screen_x,y+Screen_y);
                   vremPic.Canvas.Line(x+Screen_x,y+Screen_y,x-10+Screen_x,y+5+Screen_y);
                   vremPic.Canvas.Line(x+Screen_x,y+Screen_y,x-10+Screen_x,y-5+Screen_y);
               end;
           If arrow[multy_selected_arrow[count]].t='up' then
               begin
                   vremPic.Canvas.Line(x+Screen_x,y+Screen_y,x+Screen_x,y+30+Screen_y);
                   vremPic.Canvas.Line(x+Screen_x,y+Screen_y,x+5+Screen_x,y+10+Screen_y);
                   vremPic.Canvas.Line(x+Screen_x,y+Screen_y,x-5+Screen_x,y+10+Screen_y);
               end;
           If arrow[multy_selected_arrow[count]].t='down' then
               begin
                   vremPic.Canvas.Line(x+Screen_x,y+Screen_y,x+Screen_x,y-30+Screen_y);
                   vremPic.Canvas.Line(x+Screen_x,y+Screen_y,x+5+Screen_x,y-10+Screen_y);
                   vremPic.Canvas.Line(x+Screen_x,y+Screen_y,x-5+Screen_x,y-10+Screen_y);
               end;
       end;
   end;
   //Отрисовка дуг во фрагмент
   for count := 0 to High(multy_selected_arcs) do
   begin
       vremPic.Canvas.Pen.Color := arcs[multy_selected_arcs[count]].col;
       vremPic.Canvas.Pen.Style := arcs[multy_selected_arcs[count]].style;
       vremPic.Canvas.Pen.Width := arcs[multy_selected_arcs[count]].Width;
       With arcs[multy_selected_arcs[count]] do
            vremPic.canvas.Arc(l_x+Screen_x,t_y+Screen_y,r_x+Screen_x,
                               b_y+Screen_y,x1+Screen_x,y1+Screen_y,x2+Screen_x,y2+Screen_y);
   end;

   picX.Width := RightBorder - LeftBorder;
   picX.Height := BottomBorder - TopBorder;

   //showmessage(pic.Width.ToString+ '/' + pic.Height.ToString);

   PicX.Canvas.CopyRect(Rect(0,0,PicX.Width,PicX.Height),
                        VremPic.Canvas,
                        Rect(LeftBorder,TopBorder,RightBorder,BottomBorder));

   Clipboard.Assign(PicX);

   vremPic.Free;
   PicX.Free;

   showmessage('Фрагмент скопирован в буфер');
end;

procedure Add_vect(f_name:string); //Добавить вектор

var
  f:Text;
  s:string;

  ar5:array of string;
  i:integer;

begin
     AssignFile(f,f_name);
     Reset(f);
         Readln(f,s);//Первый заголовок

         i:=k;
         While (true) do //Загрузка линий
         begin
           Readln(f,s);
           if s= 'Circles' then break;

           ar5:=SplitString(s,'/');
           Lines[i].x1:=StrToInt(ar5[0]);
           Lines[i].y1:=StrToInt(ar5[1]);
           Lines[i].x2:=StrToInt(ar5[2]);
           Lines[i].y2:=StrToInt(ar5[3]);
           Lines[i].col:=StrToInt(ar5[4]);
           Lines[i].Width:=StrToInt(ar5[5]);
           case ar5[6] of
            'psSolid' : Lines[i].style := psSolid;
            'psDash' : Lines[i].style := psDash;
            'psDot' : Lines[i].style := psDot;
            'psDashDot' : Lines[i].style := psDashDot;
            'psDashDotDot' : Lines[i].style := psDashDotDot;
           end;

           i+=1;
         end;
         k:=i;

         i:=k_cir;
         While (true) do  //Загрузка окружностей
         begin
           Readln(f,s);
           if s = 'Arcs' then break;

           ar5:=SplitString(s,'/');
           Circles[i].x:=StrToInt(ar5[0]);
           Circles[i].y:=StrToInt(ar5[1]);
           Circles[i].r:=StrToInt(ar5[2]);
           Circles[i].col:=StrToInt(ar5[3]);
           Circles[i].Width:=StrToInt(ar5[4]);
           case ar5[5] of
            'psSolid' : Circles[i].style := psSolid;
            'psDash' : Circles[i].style := psDash;
            'psDot' : Circles[i].style := psDot;
            'psDashDot' : Circles[i].style := psDashDot;
            'psDashDotDot' : Circles[i].style := psDashDotDot;
           end;
           i+=1;
         end;
         k_cir:=i;

         i:=k_arc;
         While (true) do  //Загрузка дуг
         begin
           Readln(f,s);
           if s = 'Points' then break;

           ar5:=SplitString(s,'/');
           Arcs[i].x1:=StrToInt(ar5[0]);
           Arcs[i].y1:=StrToInt(ar5[1]);
           Arcs[i].x2:=StrToInt(ar5[2]);
           Arcs[i].y2:=StrToInt(ar5[3]);

           Arcs[i].l_x:=StrToInt(ar5[4]);
           Arcs[i].r_x:=StrToInt(ar5[5]);
           Arcs[i].t_y:=StrToInt(ar5[6]);
           Arcs[i].b_y:=StrToInt(ar5[7]);

           Arcs[i].col:=StrToInt(ar5[8]);
           Arcs[i].Width:=StrToInt(ar5[9]);

           case ar5[10] of
            'psSolid' : Circles[i].style := psSolid;
            'psDash' : Circles[i].style := psDash;
            'psDot' : Circles[i].style := psDot;
            'psDashDot' : Circles[i].style := psDashDot;
            'psDashDotDot' : Circles[i].style := psDashDotDot;
           end;
           i+=1;
         end;
         k_arc:=i;

         i:=k_points;
         While (true) do  //Загрузка точек
         begin
           Readln(f,s);
           if s = 'Texts' then break;

           ar5:=SplitString(s,'/');
           points[i].x:=StrToInt(ar5[0]);
           points[i].y:=StrToInt(ar5[1]);
           i+=1;
         end;
         k_points:=i;

         i:=k_nadp;
         While (true) do  //Загрузка текстов
         begin
           Readln(f,s);
           if s = 'Arrow' then break;

           ar5:=SplitString(s,'/');
           nadp[i].x:=StrToInt(ar5[0]);
           nadp[i].y:=StrToInt(ar5[1]);
           nadp[i].h:=StrToInt(ar5[2]);
           nadp[i].s:=ar5[3];
           nadp[i].col:=StrToInt(ar5[4]);
           i+=1;
         end;
         k_nadp:=i;

         i:=k_arrow;
         While (true) do  //Загрузка стрелок
         begin
           Readln(f,s);
           if s = 'Size' then break;

           ar5:=SplitString(s,'/');
           arrow[i].x:=StrToInt(ar5[0]);
           arrow[i].y:=StrToInt(ar5[1]);
           arrow[i].t:=ar5[2];
           arrow[i].col:=StrToInt(ar5[3]);
           i+=1;
         end;
         k_arrow:=i;

         Readln(f,loc);Readln(f,loc);//Пропуск размера чертежа

         Readln(f,loc); //Загрузка имени файла подложки

         CloseFile(f);

          If loc<>'' then //Загрузка подложки
           begin
               back_png:=TPortableNetworkGraphic.Create;
               back_png.LoadFromFile(ExtractFilePath(f_name)+loc);
               podl:=true;
               Form1.Clear_podl.Enabled:=true;
           end;
end;

procedure Load_vect(f_name_op:string); //Загрузить файл

var
   f:Text;
   s:string;

   ar5:array of string;
   i:integer;

begin
     Clear_all_fig();

     AssignFile(f,f_name_op);
     Reset(f);
     Readln(f,s);//Первый заголовок

     i:=0;
     While (true) do //Загрузка линий
     begin
       Readln(f,s);
          if s= 'Circles' then break;
       ar5:=SplitString(s,'/');
       Lines[i].x1:=StrToInt(ar5[0]);
       Lines[i].y1:=StrToInt(ar5[1]);
       Lines[i].x2:=StrToInt(ar5[2]);
       Lines[i].y2:=StrToInt(ar5[3]);
       Lines[i].col:=StrToInt(ar5[4]);
       Lines[i].Width:=StrToInt(ar5[5]);
         case ar5[6] of
          'psSolid' : Lines[i].style := psSolid;
          'psDash' : Lines[i].style := psDash;
          'psDot' : Lines[i].style := psDot;
          'psDashDot' : Lines[i].style := psDashDot;
          'psDashDotDot' : Lines[i].style := psDashDotDot;
         end;

       i+=1;
     end;
     k:=i;

     i:=0;
     While (true) do  //Загрузка окружностей
     begin
       Readln(f,s);
          if s= 'Arcs' then break;
       ar5:=SplitString(s,'/');
       Circles[i].x:=StrToInt(ar5[0]);
       Circles[i].y:=StrToInt(ar5[1]);
       Circles[i].r:=StrToInt(ar5[2]);
       Circles[i].col:=StrToInt(ar5[3]);
       Circles[i].Width:=StrToInt(ar5[4]);
        case ar5[5] of
         'psSolid' : Circles[i].style := psSolid;
         'psDash' : Circles[i].style := psDash;
         'psDot' : Circles[i].style := psDot;
         'psDashDot' : Circles[i].style := psDashDot;
         'psDashDotDot' : Circles[i].style := psDashDotDot;
        end;
       i+=1;
     end;
     k_cir:=i;

     i:=0;
     While (true) do  //Загрузка дуг
     begin
       Readln(f,s);
       if s = 'Points' then break;

       ar5:=SplitString(s,'/');
       Arcs[i].x1:=StrToInt(ar5[0]);
       Arcs[i].y1:=StrToInt(ar5[1]);
       Arcs[i].x2:=StrToInt(ar5[2]);
       Arcs[i].y2:=StrToInt(ar5[3]);

       Arcs[i].l_x:=StrToInt(ar5[4]);
       Arcs[i].r_x:=StrToInt(ar5[5]);
       Arcs[i].t_y:=StrToInt(ar5[6]);
       Arcs[i].b_y:=StrToInt(ar5[7]);

       Arcs[i].col:=StrToInt(ar5[8]);
       Arcs[i].Width:=StrToInt(ar5[9]);

       case ar5[10] of
        'psSolid' : Circles[i].style := psSolid;
        'psDash' : Circles[i].style := psDash;
        'psDot' : Circles[i].style := psDot;
        'psDashDot' : Circles[i].style := psDashDot;
        'psDashDotDot' : Circles[i].style := psDashDotDot;
       end;
       i+=1;
     end;
     k_arc:=i;

     i:=0;
     While (true) do  //Загрузка точек
     begin
       Readln(f,s);
          if s = 'Texts' then break;
       ar5:=SplitString(s,'/');
       points[i].x:=StrToInt(ar5[0]);
       points[i].y:=StrToInt(ar5[1]);
       i+=1;
     end;
     k_points:=i;

     i:=0;
     While (true) do  //Загрузка текстов
     begin
       Readln(f,s);
          if s = 'Arrow' then break;
       ar5:=SplitString(s,'/');
       nadp[i].x:=StrToInt(ar5[0]);
       nadp[i].y:=StrToInt(ar5[1]);
       nadp[i].h:=StrToInt(ar5[2]);
       nadp[i].s:=ar5[3];
       nadp[i].col:=StrToInt(ar5[4]);
       i+=1;
     end;
     k_nadp:=i;

     i:=0;
     While (true) do  //Загрузка стрелок
     begin
       Readln(f,s);
          if s = 'Size' then break;
       ar5:=SplitString(s,'/');
       arrow[i].x:=StrToInt(ar5[0]);
       arrow[i].y:=StrToInt(ar5[1]);
       arrow[i].t:=ar5[2];
       arrow[i].col:=StrToInt(ar5[3]);
       i+=1;
     end;
     k_arrow:=i;
     ///////////////////////////////////////////////////
     Readln(f,s); //Загрузка размера рабочего поля
     ar5:=SplitString(s,'/');
     Form1.WS.Width := StrToInt(ar5[0]);
     Form1.WS.Height := StrToInt(ar5[1]);
     Form1.Scroll_h.Position := 0;
     Form1.Scroll_v.Position := 0;

      if Form1.WS.Width-Form1.Panel1.Width > 0 then
         begin
              Form1.Scroll_h.Max := Form1.WS.Width-Form1.Panel1.Width;
              Form1.Scroll_h.Enabled := true;
         end
      else
        Form1.Scroll_h.Enabled:=false;
      if Form1.WS.Height-Form1.Panel1.Height > 0 then
         begin
              Form1.Scroll_v.Max := Form1.WS.Height-Form1.Panel1.Height;
              Form1.Scroll_v.Enabled := true;
         end
      else
        Form1.Scroll_v.Enabled := false;

     ////////////////////////////////////////////////////

     Readln(f,loc); //Загрузка подложки
     Readln(f,loc);

     CloseFile(f);
     f_name:=f_name_op;

    If loc<>'' then //Загрузка подложки
     begin
         back_png:=TPortableNetworkGraphic.Create;
         back_png.LoadFromFile(ExtractFilePath(f_name)+loc);
         podl:=true;
     end;

    Form1.Start_timer_delay.Enabled:=true;
end;

procedure Save_vect(f_name:string);//Сохранить вектор

var
  f:Text;
  count:integer;

begin
     AssignFile(f,f_name);
     Rewrite(f);

     Writeln(f,'Lines');
     for count:=0 to k-1 do
     Writeln(f,Lines[count].x1,'/',Lines[count].y1,'/',
               Lines[count].x2,'/',Lines[count].y2,'/',
               Lines[count].col,'/',Lines[count].Width,'/',
               Lines[count].style);

     Writeln(f,'Circles');
     for count:=0 to k_cir-1 do
     Writeln(f,Circles[count].x,'/',Circles[count].y,'/',
               Circles[count].r,'/',Circles[count].col,'/',
               Circles[count].Width,'/',Circles[count].style);

     Writeln(f,'Arcs');
     for count:=0 to k_arc-1 do
     Writeln(f,Arcs[count].x1,'/',Arcs[count].y1,'/',
               Arcs[count].x2,'/',Arcs[count].y2,'/',
               Arcs[count].l_x,'/',Arcs[count].r_x,'/',
               Arcs[count].t_y,'/',Arcs[count].b_y,'/',
               Arcs[count].col,'/',
               Arcs[count].Width,'/',Arcs[count].style);

     Writeln(f,'Points');
     for count:=0 to k_points-1 do
     Writeln(f,points[count].x,'/',points[count].y);

     Writeln(f,'Texts');
     for count:=0 to k_nadp-1 do
     Writeln(f,nadp[count].x,'/',nadp[count].y,'/',
               nadp[count].h,'/',nadp[count].s,'/',nadp[count].col);

     Writeln(f,'Arrow');
     for count:=0 to k_arrow-1 do
     Writeln(f,arrow[count].x,'/',arrow[count].y,'/',
               arrow[count].t,'/',arrow[count].col);

     Writeln(f,'Size');
     Writeln(f,IntToStr(Form1.WS.Width),'/',IntToStr(Form1.WS.Height));

     Writeln(f,'Back');

     If podl = true then
        Writeln(f,ExtractFileName(F_name+'.png')); //Название файла подложки

     CloseFile(f);

     Form1.add_back.Enabled:=true;
end;


procedure SaveScript(); //Сохранить скрипт VB-Nanocad

var
f:Text;
count:integer;
x_center, y_center, rad : integer;

begin
   AssignFile(f,DateToStr(Now)+'.vbs');
   Rewrite(f);

   //Отрезки
   Writeln(f,'Dim lineObj');
   for count:=0 to k-1 do
   Writeln(f,'Set lineObj = ThisDrawing.ModelSpace.AddLine("',
   Lines[count].x1,',',Form1.WS.Height-Lines[count].y1,',','0", "',
   Lines[count].x2,',',Form1.WS.Height-Lines[count].y2,',','0")');

   //Окружности
   Writeln(f,'Dim circObj');
   for count:=0 to k_cir-1 do
   Writeln(f,'Set circObj = ThisDrawing.ModelSpace.AddCircle("',
   Circles[count].x,',',Form1.WS.Height-Circles[count].y,',0",',Circles[count].r,')');

   //Дуги (вместо дуг будут окружности)
   Writeln(f,'Dim ArcObj');
   for count:=0 to k_arc-1 do
   begin
       x_center := (arcs[count].l_x + arcs[count].r_x) div 2;
       y_center := (arcs[count].b_y + arcs[count].t_y) div 2;
       rad := abs((arcs[count].l_x-arcs[count].r_x) div 2);

       Writeln(f,'Set ArcObj = ThisDrawing.ModelSpace.AddCircle("',
       x_center,',',Form1.WS.Height-y_center,',0",',rad,')');
    end;

   //Текст
   Writeln(f,'Dim textObj');
   for count:=0 to k_nadp-1 do
   Writeln(f,'Set textObj = ThisDrawing.ModelSpace.AddText("',Nadp[count].s,'","',
   Nadp[count].x,',',Form1.WS.Height-Nadp[count].y-Nadp[count].h,',','0"',',',Nadp[count].h,')');

   //Стрелки
   Writeln(f,'Dim ArrowObj');
   for count:=0 to k_arrow-1 do
   begin
        case Arrow[count].t of
          'down':
              begin
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y+30,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x+5,',',Form1.WS.Height-Arrow[count].y+5,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x-5,',',Form1.WS.Height-Arrow[count].y+5,',','0")');
              end;
          ///
          'up':
              begin
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y-30,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x+5,',',Form1.WS.Height-Arrow[count].y-5,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x-5,',',Form1.WS.Height-Arrow[count].y-5,',','0")');
              end;
          'right':
              begin
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x-30,',',Form1.WS.Height-Arrow[count].y,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x-5,',',Form1.WS.Height-Arrow[count].y-5,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x-5,',',Form1.WS.Height-Arrow[count].y+5,',','0")');
              end;
          'left':
              begin
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x+30,',',Form1.WS.Height-Arrow[count].y,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x+5,',',Form1.WS.Height-Arrow[count].y+5,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x+5,',',Form1.WS.Height-Arrow[count].y-5,',','0")');
              end;
          end;
   end;
   Close(f);
end;

end.