unit my_modify;

{
    Vector graphical editor.
    Version: 1.
    Written on FreePascal (https://freepascal.org/).
    Copyright (C) 2023  Warfalomay

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
}

{$mode ObjFPC}{$H+}

interface
//Сдвинуть один или несколько объектов
procedure Move_vect(side_x, side_y : integer);
//Сохранить предыдущий шаг
procedure save_prev();
//Удалить объект/объекты
procedure Delete_Ob();
//Удалить всё
procedure Clear_all_fig();
//Выбор объекта
function select(x,y:integer):integer;
//Объектная привязка
procedure near_points(var x,y : integer; dist : integer);
//Поворот выбраной линии
procedure Rotate(xr,yr : integer);
//Перетаскивание за точку
procedure Drag_by_point(x,y : integer);
//Выбор объектов рамкой
procedure SelectByFrame(x1,y1,x2,y2 : integer);


var
  block_num : integer; //Переменная для выделения блока

implementation
uses Graphics,Unit1,Classes, SysUtils, View, math, Dialogs, Controls, MyFunc;


procedure Drag_by_point(x,y : integer);//Перетаскивание за точку
begin
    If select_type = 1 then //Если выбрана линия
    begin
       If ((x <> lines[Selected].x1) and //Если указал не конечную точку
          (y <> lines[Selected].y1)) and
          ((x <> lines[Selected].x2) and
          (y <> lines[Selected].y2)) then
          exit;
       Form1.WS.Cursor := crHandPoint;
       drag_point_mode := true;
       Drag_Point.X := x;
       Drag_Point.Y := y;
       Drag_selected := Selected;
    end;
    If select_type = 2 then //Если выбрана окружность
    begin
       If ((x <> circles[Selected].x) and //Если указал не конечную точку
          (y <> circles[Selected].y)) then
          exit;
       Form1.WS.Cursor := crHandPoint;
       drag_point_mode := true;
       Drag_Point.X := x;
       Drag_Point.Y := y;
       Drag_selected := Selected;
    end;
end;

function CrossLine(num,x1,x2,y1,y2 : integer) : boolean;

var
  v1,v2,v3,v4:real;

begin
  Result:=false;

  v1:=(Lines[num].x2-x1)*(y1-Lines[num].y1)-(Lines[num].y2-Lines[num].y1)*(x1-Lines[num].x1);
  v2:=(Lines[num].x2-Lines[num].x1)*(y2-Lines[num].y1)-(Lines[num].y2-Lines[num].y1)*(x2-Lines[num].x1);
  v3:=(x2-x1)*(Lines[num].y1-y1)-(y2-y1)*(Lines[num].x1-x1);
  v4:=(x2-x1)*(Lines[num].y2-y1)-(y2-y1)*(Lines[num].x2-x1);
  If (v1*v2<0) and (v3*v4<0) then result:=true;// else Intersection:='no';
end;


procedure SelectByFrame(x1,y1,x2,y2 : integer); //Выбор объектов рамкой

var
  count : integer;
  Xmin, Ymin : integer;
  Xmax, Ymax : integer;

begin
    Clear;

    //Clear();
    //selected:=Select(x,y);

    Xmin := min(x1,x2);
    Ymin := min(y1,y2);
    Xmax := max(x1,x2);
    Ymax := max(y1,y2);

    //Проверка выбора линий
    for count := 0 to High(lines) do
    begin
        if ((lines[count].x1 >= Xmin) and
           (lines[count].x1 <= Xmax) and
           (lines[count].y1 >= Ymin) and
           (lines[count].y1 <= Ymax)) or
           ((lines[count].x2 >= Xmin) and
           (lines[count].x2 <= Xmax) and
           (lines[count].y2 >= Ymin) and
           (lines[count].y2 <= Ymax)) or
           (CrossLine(count,x1,x2,y1,y2))then
           begin
             setLength(multy_selected_line,Length(multy_selected_line)+1);
             multy_selected_line[High(multy_selected_line)] := count;
             Form1.WS.Canvas.Pen.Color := clRed;
             Form1.WS.Canvas.Line(lines[count].x1 + Screen_x,lines[count].y1 + Screen_y,
                                  lines[count].x2 + Screen_x,lines[count].y2 + Screen_y);
           end;
    end;

    //Проверка выбора окружностей
    for count := 0 to High(circles) do
    begin
        if ((circles[count].x >= Xmin) and
           (circles[count].x <= Xmax) and
           (circles[count].y >= Ymin) and
           (circles[count].y <= Ymax)) or
           ((Xmin - circles[count].x <= circles[count].r) and
           (Ymin - circles[count].y <= circles[count].r)) or
           ((Xmax - circles[count].x <= circles[count].r) and
           (Ymax - circles[count].y <= circles[count].r)) then
           begin
             setLength(multy_selected_circle,Length(multy_selected_circle)+1);
             multy_selected_circle[High(multy_selected_circle)] := count;
             Form1.WS.Canvas.Pen.Color := clRed;
             Form1.WS.Canvas.Ellipse(circles[count].x + Screen_x - circles[count].r,
                                     circles[count].y + Screen_y - circles[count].r,
                                     circles[count].x + Screen_x + circles[count].r,
                                     circles[count].y + Screen_y + circles[count].r);
           end;
    end;

    //Проверка выбора точек
    If P_Show then
    for count := 0 to High(points) do
    begin
        if ((points[count].x >= Xmin) and
           (points[count].x <= Xmax) and
           (points[count].y >= Ymin) and
           (points[count].y <= Ymax)) then
           begin
             setLength(multy_selected_point,Length(multy_selected_point)+1);
             multy_selected_point[High(multy_selected_point)] := count;
             Form1.WS.Canvas.Pen.Color := clRed;

             Form1.WS.Canvas.Ellipse(points[count].x + Screen_x - 5,
                                     points[count].y + Screen_y - 5,
                                     points[count].x + Screen_x + 5,
                                     points[count].y + Screen_y + 5);
           end;
    end;

    //Проверка выбора текста
    for count := 0 to High(points) do
    begin
        if ((nadp[count].x >= Xmin) and
           (nadp[count].x <= Xmax) and
           (nadp[count].y >= Ymin) and
           (nadp[count].y <= Ymax)) then
           begin
             setLength(multy_selected_text,Length(multy_selected_text)+1);
             multy_selected_text[High(multy_selected_text)] := count;
             Form1.WS.Canvas.Font.Color := clRed;
             Form1.WS.Canvas.Font.Size := nadp[count].h;
             Form1.WS.Canvas.TextOut(nadp[count].x + Screen_x, nadp[count].y + Screen_y,nadp[count].s);
           end;
    end;

    //Проверка выбора стрелок
    for count := 0 to High(points) do
    begin
        if ((arrow[count].x >= Xmin) and
           (arrow[count].x <= Xmax) and
           (arrow[count].y >= Ymin) and
           (arrow[count].y <= Ymax)) then
           begin
             setLength(multy_selected_arrow,Length(multy_selected_arrow)+1);
             multy_selected_arrow[High(multy_selected_arrow)] := count;
             Form1.WS.Canvas.Pen.Color := clRed;

             Form1.WS.Canvas.Ellipse(arrow[count].x + Screen_x -5,arrow[count].y + Screen_y +5,
                                        arrow[count].x + Screen_x +5,arrow[count].y + + Screen_y -5);
             if arrow[count].t = 'left' then
                  Form1.WS.Canvas.Ellipse(arrow[count].x+30-5+Screen_x,arrow[count].y+5+Screen_y,
                                          arrow[count].x+30+5+Screen_x,arrow[count].y-5+Screen_y);
             if arrow[count].t = 'right' then
                  Form1.WS.Canvas.Ellipse(arrow[count].x-30-5+Screen_x,arrow[count].y+5+Screen_y,
                                          arrow[count].x-30+5+Screen_x,arrow[count].y-5+Screen_y);
             if arrow[count].t = 'up' then
                  Form1.WS.Canvas.Ellipse(arrow[count].x-5+Screen_x,arrow[count].y+30+5+Screen_y,
                                          arrow[count].x+5+Screen_x,arrow[count].y+30-5+Screen_y);
             if arrow[count].t = 'down' then
                  Form1.WS.Canvas.Ellipse(arrow[count].x-5+Screen_x,arrow[count].y-30+5+Screen_y,
                                              arrow[count].x+5+Screen_x,arrow[count].y-30-5+Screen_y);
           end;
    end;

    //Проверка выбора дуги
    for count := 0 to High(points) do
    begin
        if ((arcs[count].x1 >= Xmin) and
           (arcs[count].x2 <= Xmax) and
           (arcs[count].y1 >= Ymin) and
           (arcs[count].y2 <= Ymax)) then
           begin
             setLength(multy_selected_arcs,Length(multy_selected_arcs)+1);
             multy_selected_arcs[High(multy_selected_arcs)] := count;
             Form1.WS.Canvas.Pen.Color := clRed;

             Form1.WS.canvas.Arc(Arcs[count].l_x+Screen_x,Arcs[count].t_y+Screen_y,
                             Arcs[count].r_x+Screen_x,Arcs[count].b_y+Screen_y,
                             Arcs[count].x1+Screen_x,Arcs[count].y1+Screen_y,Arcs[count].x2+Screen_x,Arcs[count].y2+Screen_y);
           end;
    end;

    if (Length(multy_selected_line)<>0) or
      (Length(multy_selected_circle)<>0) or
      (Length(multy_selected_arcs)<>0) or
      (Length(multy_selected_point)<>0) or
      (Length(multy_selected_text)<>0) or
      (Length(multy_selected_arrow)<>0) then
      Form1.StatusBar1.Panels.Items[0].Text:='Выбраны элементы'
    else
      Form1.StatusBar1.Panels.Items[0].Text:='Ничего не выбрано';

    Point1.X := -1;
    Point1.Y := -1;
end;

procedure near_points(var x,y : integer; dist:integer);//Объектная привязка

var
  count:integer;

begin
  Form1.WS.Cursor := crDefault;

  for count:=0 to Arr_length-1 do
  begin
    //Проверка по массиву линий
    if (sqrt(sqr(abs(x-Lines[count].x1))+sqr(abs(y-Lines[count].y1)))<dist) then
     begin
       //if count = Drag_selected then continue;
       x:=Lines[count].x1;
       y:=Lines[count].y1;
       Form1.WS.Cursor := crCross;
     end;
    if (sqrt(sqr(abs(x-Lines[count].x2))+sqr(abs(y-Lines[count].y2)))<dist) then
     begin
       //if count = Drag_selected then continue;
       x:=Lines[count].x2;
       y:=Lines[count].y2;
       Form1.WS.Cursor := crCross;
     end;


    //Проверка по массиву кругов////////////////////////////////////////////////////////
    //Центр
    if (sqrt(sqr(abs(x-Circles[count].x))+sqr(abs(y-Circles[count].y)))<dist) then
     begin
       //if count = Drag_selected then continue;
       x:=Circles[count].x;
       y:=Circles[count].y;
       Form1.WS.Cursor := crCross;
     end;
    //Лев.орд.
    if (sqrt(sqr((x-(Circles[count].x-Circles[count].r)))+sqr(abs(y-Circles[count].y)))<dist) then
     begin
       //if count = Drag_selected then continue;
       x:=Circles[count].x-Circles[count].r;
       y:=Circles[count].y;
       Form1.WS.Cursor := crCross;
     end;
    //Прав.орд.
    if (sqrt(sqr((x-(Circles[count].x+Circles[count].r)))+sqr(abs(y-Circles[count].y)))<dist) then
     begin
       //if count = Drag_selected then continue;
       x:=Circles[count].x+Circles[count].r;
       y:=Circles[count].y;
       Form1.WS.Cursor := crCross;
     end;
    //Верх.орд.
    if (sqrt(sqr(abs(x-Circles[count].x))+sqr((y-Circles[count].y-Circles[count].r)))<dist) then
     begin
       //if count = Drag_selected then continue;
       x:=Circles[count].x;
       y:=Circles[count].y+Circles[count].r;
       Form1.WS.Cursor := crCross;
     end;
    //Нижн.орд.
    if (sqrt(sqr(abs(x-Circles[count].x))+sqr((y-Circles[count].y+Circles[count].r)))<dist) then
     begin
       //if count = Drag_selected then continue;
       x:=Circles[count].x;
       y:=Circles[count].y-Circles[count].r;
       Form1.WS.Cursor := crCross;
     end;


    //Проверка по массиву дуг////////////////////////////////////////////////////////////////
    if (sqrt(sqr(abs(x-Arcs[count].x1))+sqr(abs(y-Arcs[count].y1)))<dist) then
     begin
       //if count = Drag_selected then continue;
       x:=Arcs[count].x1;
       y:=Arcs[count].y1;
       Form1.WS.Cursor := crCross;
     end;
    if (sqrt(sqr(abs(x-Arcs[count].x2))+sqr(abs(y-Arcs[count].y2)))<dist) then
     begin
       //if count = Drag_selected then continue;
       x:=Arcs[count].x2;
       y:=Arcs[count].y2;
       Form1.WS.Cursor := crCross;
     end;

    //Проверка по массиву точек
    if (sqrt(sqr(abs(x-points[count].x))+sqr(abs(y-points[count].y)))<dist) then
     begin
       //if count = Drag_selected then continue;
       x:=points[count].x;
       y:=points[count].y;
       Form1.WS.Cursor := crCross;
     end;

  end;
end;

procedure save_prev(); //Сохранить предыдущий шаг
begin
   stad:=false;

   lines_vrem:=copy(lines);
   circles_vrem:=copy(circles);
   arcs_vrem:=copy(arcs);
   points_vrem:=copy(points);
   nadp_vrem:=copy(nadp);
   arrow_vrem:=copy(arrow);

   k_vrem:=k;
   k_cir_vrem:=k_cir;
   k_arc_vrem:=k_arc;
   k_points_vrem:=k_points;
   k_nadp_vrem:=k_nadp;
   k_arrow_vrem:=k_arrow;
end;

procedure Move_vect(side_x, side_y : integer); //Сдвинуть один или несколько объектов

var
 i:integer;
 count : integer;

begin
  //Если выбрано несколько объектов ///////////////////////////////////////////
 if (Length(multy_selected_line)<>0) or
    (Length(multy_selected_circle)<>0) or
    (Length(multy_selected_arcs)<>0) or
    (Length(multy_selected_point)<>0) or
    (Length(multy_selected_text)<>0) or
    (Length(multy_selected_arrow)<>0) then
 begin
    save_prev(); //Сохранить предыдущее состояние

    for count := 0 to High(multy_selected_line) do
    begin
       Lines[multy_selected_line[count]].x1+=side_x;
       Lines[multy_selected_line[count]].x2+=side_x;
       Lines[multy_selected_line[count]].y1+=side_y;
       Lines[multy_selected_line[count]].y2+=side_y;
    end;
    for count := 0 to High(multy_selected_circle) do
    begin
       circles[multy_selected_circle[count]].x+=side_x;
       circles[multy_selected_circle[count]].y+=side_y;
    end;
    for count := 0 to High(multy_selected_arcs) do
    begin
       arcs[multy_selected_arcs[count]].x1+=side_x;
       arcs[multy_selected_arcs[count]].y1+=side_y;
       arcs[multy_selected_arcs[count]].x2+=side_x;
       arcs[multy_selected_arcs[count]].y2+=side_y;
       arcs[multy_selected_arcs[count]].l_x+=side_x;
       arcs[multy_selected_arcs[count]].r_x+=side_y;
       arcs[multy_selected_arcs[count]].t_y+=side_x;
       arcs[multy_selected_arcs[count]].b_y+=side_y;
    end;
    for count := 0 to High(multy_selected_point) do
    begin
       points[multy_selected_point[count]].x+=side_x;
       points[multy_selected_point[count]].y+=side_y;
    end;
    for count := 0 to High(multy_selected_text) do
    begin
       nadp[multy_selected_text[count]].x+=side_x;
       nadp[multy_selected_text[count]].y+=side_y;
    end;
    for count := 0 to High(multy_selected_arrow) do
    begin
       arrow[multy_selected_arrow[count]].x+=side_x;
       arrow[multy_selected_arrow[count]].y+=side_y;
    end;
    Form1.Start_timer_delay.Enabled:=true;
    exit;
 end;

 //Если выбран один объект ////////////////////////////////////
  If (select_type<>10000) and not ((select_type >= 1000000) and
                                   (select_type <= 1111111)) then
 begin
   save_prev(); //Сохранить предыдущее состояние
   if Select_type=1 then
    begin
      lines[selected].x1+=side_x;
      lines[selected].x2+=side_x;
      lines[selected].y1+=side_y;
      lines[selected].y2+=side_y;
    end;
   if Select_type=2 then
    begin
      Circles[selected].x+=side_x;
      Circles[selected].y+=side_y;
    end;
   if Select_type=3 then
    begin
      points[selected].x+=side_x;
      points[selected].y+=side_y;
    end;
    if Select_type=4 then
    begin
      nadp[selected].x+=side_x;
      nadp[selected].y+=side_y;
    end;
    if Select_type=5 then
    begin
      arrow[selected].x+=side_x;
      arrow[selected].y+=side_y;
    end;
     if Select_type=6 then
    begin
      arcs[selected].x1+=side_x;
      arcs[selected].y1+=side_y;
      arcs[selected].x2+=side_x;
      arcs[selected].y2+=side_y;
      arcs[selected].r_x+=side_x;
      arcs[selected].l_x+=side_x;
      arcs[selected].t_y+=side_y;
      arcs[selected].b_y+=side_y;
    end;
    Form1.Start_timer_delay.Enabled:=true;
    Exit;
 end;


  //Сдвинуть все выбраные объекты
 If (select_type >= 1000000) and (select_type <= 1111111) then
 begin
       save_prev(); //Сохранить предыдущее состояние
       If IntToStr(select_type)[7] ='1' then  //Сдвинуть все линии
       begin

        for i:=k downto 0 do
         begin
           lines[i].x1+=side_x;
           lines[i].x2+=side_x;
           lines[i].y1+=side_y;
           lines[i].y2+=side_y;
          end;
       end;

       If IntToStr(select_type)[6] ='1' then   //Сдвинуть все окружности
       begin
        for i:=k_cir downto 0 do
         begin
           Circles[i].x+=side_x;
           Circles[i].y+=side_y;
          end;
       end;

       If IntToStr(select_type)[5] ='1' then   //Сдвинуть все точки
       begin
        for i:=k_points downto 0 do
         begin
           If (points[i].x<>0) and (points[i].y<>0) then
           begin
               points[i].x+=side_x;
               points[i].y+=side_y;
           end;
          end;
       end;

       If IntToStr(select_type)[4] ='1' then   //Сдвинуть все тексты
       begin
        for i:=k_nadp downto 0 do
         begin
           nadp[i].x+=side_x;
           nadp[i].y+=side_y;
          end;
       end;

       If IntToStr(select_type)[3] ='1' then   //Сдвинуть все стрелки
       begin
        for i:=k_arrow downto 0 do
         begin
           arrow[i].x+=side_x;
           arrow[i].y+=side_y;
          end;
       end;

       If IntToStr(select_type)[2] ='1' then   //Сдвинуть все дуги
       begin
        for i:=k_arc downto 0 do
         begin
           arcs[i].x1+=side_x;
           arcs[i].x2+=side_x;
           arcs[i].y1+=side_y;
           arcs[i].y2+=side_y;
           arcs[i].r_x+=side_x;
           arcs[i].l_x+=side_x;
           arcs[i].t_y+=side_y;
           arcs[i].b_y+=side_y;
          end;
       end;

 end;
 Form1.Start_timer_delay.Enabled:=true;
end;

procedure Delete_Ob(); //Удалить объект

var
 i:integer;
 count : integer;

begin
  save_prev(); //Сохранить предыдущее состояние

 //Если выбрано несколько объектов
 if (Length(multy_selected_line)<>0) or
    (Length(multy_selected_circle)<>0) or
    (Length(multy_selected_arcs)<>0) or
    (Length(multy_selected_point)<>0) or
    (Length(multy_selected_text)<>0) or
    (Length(multy_selected_arrow)<>0) then
 begin
    MyFunc.sort(multy_selected_line);
    MyFunc.sort(multy_selected_circle);
    MyFunc.sort(multy_selected_arcs);
    MyFunc.sort(multy_selected_point);
    MyFunc.sort(multy_selected_text);
    MyFunc.sort(multy_selected_arrow);

    for count := High(multy_selected_line) downto 0 do
    begin
        Delete(lines,multy_selected_line[count],1);
        SetLength(lines,arr_length);
        k-=1;
    end;
    for count := High(multy_selected_circle) downto 0 do
    begin
        Delete(circles,multy_selected_circle[count],1);
        SetLength(circles,arr_length);
        k_cir-=1;
    end;
    for count := High(multy_selected_arcs) downto 0 do
    begin
        Delete(arcs,multy_selected_arcs[count],1);
        SetLength(arcs,arr_length);
        k_arc-=1;
    end;
    for count := High(multy_selected_point) downto 0 do
    begin
        Delete(points,multy_selected_point[count],1);
        SetLength(points,arr_length);
        k_points-=1;
    end;
    for count := High(multy_selected_text) downto 0 do
    begin
        Delete(nadp,multy_selected_text[count],1);
        SetLength(nadp,arr_length);
        k_nadp-=1;
    end;
    for count := High(multy_selected_arrow) downto 0 do
    begin
        Delete(arrow,multy_selected_arrow[count],1);
        SetLength(arrow,arr_length);
        k_arrow-=1;
    end;

    SetLength(multy_selected_line,0);
    SetLength(multy_selected_circle,0);
    SetLength(multy_selected_arcs,0);
    SetLength(multy_selected_point,0);
    SetLength(multy_selected_text,0);
    SetLength(multy_selected_arrow,0);

    selected := 10000;

    clear;
    exit;
 end;

 //Выбран одиночный объект ///////////////////////////////////////////////////
 If (select_type<>10000) and not ((select_type >= 1000000) and
                                 (select_type <= 1111111)) then
 begin
 if Select_type=1 then
  begin
    Delete(lines,selected,1);
    SetLength(lines,arr_length);
    k-=1;
  end;
 if Select_type=2 then
  begin
    Delete(Circles,selected,1);
    SetLength(Circles,arr_length);
    k_cir-=1;
  end;
 if Select_type=3 then
  begin
    Delete(points,selected,1);
    SetLength(points,arr_length);
    k_points-=1;
  end;
  if Select_type=4 then
  begin
    Delete(nadp,selected,1);
    SetLength(nadp,arr_length);
    k_nadp-=1;
  end;
  if Select_type=5 then  //Стрелка
  begin
    Delete(arrow,selected,1);
    SetLength(arrow,arr_length);
    k_arrow-=1;
  end;
  if Select_type=6 then  //Дуга
  begin
    Delete(arcs,selected,1);
    SetLength(arcs,arr_length);
    k_arc-=1;
  end;
  selected:=10000;
 end;

//Удалить все объекты
 If (select_type >= 1000000) and (select_type <= 1111111) then
 begin

     If IntToStr(select_type)[7] ='1' then  //Удалить все линии
     begin

      for i:=k downto 0 do
       begin
         Delete(lines,i,1);
        end;
      SetLength(lines,arr_length);
      k:=0;
     end;

     If IntToStr(select_type)[6] ='1' then   //Удалить все окружности
     begin
      for i:=k_cir downto 0 do
       begin
         Delete(Circles,i,1);
        end;
      SetLength(Circles,arr_length);
      k_cir:=0;
     end;

     If IntToStr(select_type)[5] ='1' then   //Удалить все точки
     begin
      for i:=k_points downto 0 do
       begin
         Delete(points,i,1);
        end;
      SetLength(points,arr_length);
      k_points:=0;
     end;

     If IntToStr(select_type)[4] ='1' then   //Удалить все тексты
     begin
      for i:=k_nadp downto 0 do
       begin
         Delete(nadp,i,1);
        end;
      SetLength(nadp,arr_length);
      k_nadp:=0;
     end;

     If IntToStr(select_type)[3] ='1' then   //Удалить все стрелки
     begin
      for i:=k_arrow downto 0 do
       begin
         Delete(arrow,i,1);
        end;
      SetLength(arrow,arr_length);
      k_arrow:=0;
     end;

     If IntToStr(select_type)[2] ='1' then   //Удалить все дуги
     begin
      for i:=k_arc downto 0 do
       begin
         Delete(arcs,i,1);
        end;
      SetLength(arcs,arr_length);
      k_arc:=0;
     end;

 end;

  selected:=10000;
  select_type:=0;
  Clear();
  Form1.StatusBar1.Panels.Items[0].Text:='Ничего не выбрано';

  //select_mode:= false;
  //Reset_mode;

end;

procedure Clear_all_fig(); //Удалить всё

var
  i:integer;

begin
  save_prev(); //Сохранить предыдущее состояние

  for i:=k downto 0 do
   begin
     Delete(lines,i,1);
    end;
  for i:=k_cir downto 0 do
   begin
     Delete(Circles,i,1);
   end;
  for i:=k_arc downto 0 do
   begin
     Delete(Arcs,i,1);
   end;
  for i:=k_points downto 0 do
   begin
     Delete(points,i,1);
   end;
  for i:=k_nadp downto 0 do
   begin
     Delete(nadp,i,1);
   end;
  for i:=k_arrow downto 0 do
   begin
     Delete(arrow,i,1);
   end;

  SetLength(lines,arr_length);
  SetLength(Circles,arr_length);
  SetLength(points,arr_length);
  SetLength(nadp,arr_length);
  SetLength(arrow,arr_length);
  SetLength(arcs,arr_length);

  k:=0; k_cir:=0; k_points:=0; k_nadp:=0; k_arrow:=0; k_arc:=0;

  loc:='';
  f_name:='';
  podl:=false;
  //back_png.Free;

  Form1.add_back.Enabled:=false;
  Form1.Clear_podl.Enabled:=False;

 selected:=10000;
 Clear();
 Form1.StatusBar1.Panels.Items[0].Text:='Ничего не выбрано';
end;

//Проверка по массиву блоков
function Prov_Sel_Block(num : integer):integer;

var
  count, count2 : integer;

begin
   result := -1;

   for count := 0 to High(blocks) do
    begin
        //Проверка линий
        for count2 := 0 to High(blocks[count]) do
        if ((blocks[count,count2].kind = 'ln') or
            (blocks[count,count2].kind = 'cr') or
            (blocks[count,count2].kind = 'ar') or
            (blocks[count,count2].kind = 'pt') or
            (blocks[count,count2].kind = 'tx') or
            (blocks[count,count2].kind = 'arw')) and
            (blocks[count,count2].num = num) then
               begin
                    result := count;
                    exit;
               end;
    end;
end;

procedure Select_block(block_num : integer);

var
    block_counter : integer;

begin
    Clear;
    For block_counter := 0 to High(blocks[block_num]) do
       begin
         If blocks[block_num,block_counter].kind='ln' then
         begin
             form1.WS.Canvas.Ellipse(Lines[blocks[block_num,block_counter].num].x1-5+Screen_x,Lines[blocks[block_num,block_counter].num].y1-5+screen_y,
                                     Lines[blocks[block_num,block_counter].num].x1+5+Screen_x,Lines[blocks[block_num,block_counter].num].y1+5+Screen_y);
             form1.WS.Canvas.Ellipse(Lines[blocks[block_num,block_counter].num].x2-5+Screen_x,Lines[blocks[block_num,block_counter].num].y2-5+Screen_y,
                                     Lines[blocks[block_num,block_counter].num].x2+5+Screen_x,Lines[blocks[block_num,block_counter].num].y2+5+Screen_y);
             //Занести линии в массив мультивыделения
             setLength(multy_selected_line,Length(multy_selected_line)+1);
             multy_selected_line[High(multy_selected_line)] := blocks[block_num,block_counter].num;
         end;


         If blocks[block_num,block_counter].kind='cr' then
         begin
             Form1.WS.Canvas.Ellipse(Circles[blocks[block_num,block_counter].num].x-Circles[blocks[block_num,block_counter].num].r-5+Screen_x,
                                     Circles[blocks[block_num,block_counter].num].y+Circles[blocks[block_num,block_counter].num].r+5+Screen_y,
                                     Circles[blocks[block_num,block_counter].num].x-Circles[blocks[block_num,block_counter].num].r+5+Screen_x,
                                     Circles[blocks[block_num,block_counter].num].y+Circles[blocks[block_num,block_counter].num].r-5+Screen_y);
             Form1.WS.Canvas.Ellipse(Circles[blocks[block_num,block_counter].num].x-Circles[blocks[block_num,block_counter].num].r-5+Screen_x,
                                     Circles[blocks[block_num,block_counter].num].y-Circles[blocks[block_num,block_counter].num].r+5+Screen_y,
                                     Circles[blocks[block_num,block_counter].num].x-Circles[blocks[block_num,block_counter].num].r+5+Screen_x,
                                     Circles[blocks[block_num,block_counter].num].y-Circles[blocks[block_num,block_counter].num].r-5+Screen_y);
             Form1.WS.Canvas.Ellipse(Circles[blocks[block_num,block_counter].num].x+Circles[blocks[block_num,block_counter].num].r-5+Screen_x,
                                     Circles[blocks[block_num,block_counter].num].y+Circles[blocks[block_num,block_counter].num].r+5+Screen_y,
                                     Circles[blocks[block_num,block_counter].num].x+Circles[blocks[block_num,block_counter].num].r+5+Screen_x,
                                     Circles[blocks[block_num,block_counter].num].y+Circles[blocks[block_num,block_counter].num].r-5+Screen_y);
             Form1.WS.Canvas.Ellipse(Circles[blocks[block_num,block_counter].num].x+Circles[blocks[block_num,block_counter].num].r-5+Screen_x,
                                     Circles[blocks[block_num,block_counter].num].y-Circles[blocks[block_num,block_counter].num].r+5+Screen_y,
                                     Circles[blocks[block_num,block_counter].num].x+Circles[blocks[block_num,block_counter].num].r+5+Screen_x,
                                     Circles[blocks[block_num,block_counter].num].y-Circles[blocks[block_num,block_counter].num].r-5+Screen_y);
             Form1.WS.Canvas.Ellipse(Circles[blocks[block_num,block_counter].num].x-3+Screen_x,Circles[blocks[block_num,block_counter].num].y-3+Screen_y,
                                     Circles[blocks[block_num,block_counter].num].x+3+Screen_x,Circles[blocks[block_num,block_counter].num].y+3+Screen_y);

             //Занести круг в массив мультивыделения
             setLength(multy_selected_circle,Length(multy_selected_circle)+1);
             multy_selected_circle[High(multy_selected_circle)] := blocks[block_num,block_counter].num;
         end;

         If blocks[block_num,block_counter].kind='pt' then
         begin
             Form1.WS.Canvas.Ellipse(points[blocks[block_num,block_counter].num].x-15+Screen_x,points[blocks[block_num,block_counter].num].y+15+Screen_y,
                                     points[blocks[block_num,block_counter].num].x+15+Screen_x,points[blocks[block_num,block_counter].num].y-15+Screen_y);
             //Занести точку в массив мультивыделения
             setLength(multy_selected_point,Length(multy_selected_point)+1);
             multy_selected_point[High(multy_selected_point)] := blocks[block_num,block_counter].num;
         end;

         If blocks[block_num,block_counter].kind='ar' then
         begin
             Form1.WS.Canvas.Ellipse(Arcs[blocks[block_num,block_counter].num].x1-5+Screen_x,
                                     Arcs[blocks[block_num,block_counter].num].y1-5+Screen_y,
                                     Arcs[blocks[block_num,block_counter].num].x1+5+Screen_x,
                                     Arcs[blocks[block_num,block_counter].num].y1+5+Screen_y);
             Form1.WS.Canvas.Ellipse(Arcs[blocks[block_num,block_counter].num].x2-5+Screen_x,
                                     Arcs[blocks[block_num,block_counter].num].y2-5+Screen_y,
                                     Arcs[blocks[block_num,block_counter].num].x2+5+Screen_x,
                                     Arcs[blocks[block_num,block_counter].num].y2+5+Screen_y);
             //Занести дугу в массив мультивыделения
             setLength(multy_selected_arcs,Length(multy_selected_arcs)+1);
             multy_selected_arcs[High(multy_selected_arcs)] := blocks[block_num,block_counter].num;
         end;

         If blocks[block_num,block_counter].kind='tx' then
         begin
             Form1.WS.Canvas.Font.Size:=nadp[blocks[block_num,block_counter].num].h;
             Form1.WS.Canvas.Font.Color:= clRed;
             Form1.WS.Canvas.TextOut(nadp[blocks[block_num,block_counter].num].x+Screen_x,nadp[blocks[block_num,block_counter].num].y+Screen_y,
                                     nadp[blocks[block_num,block_counter].num].s);
             //Занести текст в массив мультивыделения
             setLength(multy_selected_text,Length(multy_selected_text)+1);
             multy_selected_text[High(multy_selected_text)] := blocks[block_num,block_counter].num;
         end;

         If blocks[block_num,block_counter].kind='arw' then
         begin
            Form1.WS.Canvas.Ellipse(arrow[blocks[block_num,block_counter].num].x-5,arrow[blocks[block_num,block_counter].num].y+5,
                                    arrow[blocks[block_num,block_counter].num].x+5,arrow[blocks[block_num,block_counter].num].y-5);
            if arrow[blocks[block_num,block_counter].num].t = 'left' then
                  Form1.WS.Canvas.Ellipse(arrow[blocks[block_num,block_counter].num].x+30-5+Screen_x,arrow[blocks[block_num,block_counter].num].y+5+Screen_y,
                                          arrow[blocks[block_num,block_counter].num].x+30+5+Screen_x,arrow[blocks[block_num,block_counter].num].y-5+Screen_y);
            if arrow[blocks[block_num,block_counter].num].t = 'right' then
                  Form1.WS.Canvas.Ellipse(arrow[blocks[block_num,block_counter].num].x-30-5+Screen_x,arrow[blocks[block_num,block_counter].num].y+5+Screen_y,
                                          arrow[blocks[block_num,block_counter].num].x-30+5+Screen_x,arrow[blocks[block_num,block_counter].num].y-5+Screen_y);
            if arrow[blocks[block_num,block_counter].num].t = 'up' then
                  Form1.WS.Canvas.Ellipse(arrow[blocks[block_num,block_counter].num].x-5+Screen_x,arrow[blocks[block_num,block_counter].num].y+30+5+Screen_y,
                                          arrow[blocks[block_num,block_counter].num].x+5+Screen_x,arrow[blocks[block_num,block_counter].num].y+30-5+Screen_y);
            if arrow[blocks[block_num,block_counter].num].t = 'down' then
                  Form1.WS.Canvas.Ellipse(arrow[blocks[block_num,block_counter].num].x-5+Screen_x,arrow[blocks[block_num,block_counter].num].y-30+5+Screen_y,
                                          arrow[blocks[block_num,block_counter].num].x+5+Screen_x,arrow[blocks[block_num,block_counter].num].y-30-5+Screen_y);

            //Занести стрелку в массив мультивыделения
             setLength(multy_selected_arrow,Length(multy_selected_arrow)+1);
             multy_selected_arrow[High(multy_selected_arrow)] := blocks[block_num,block_counter].num;
         end;

       end;
end;

function select(x,y:integer):integer;//Выбор объекта

var
  count:integer;
  a,b,c:real;
  p:real;
  h:integer = 0;
  alpha,beta,gamma:real;
  angle_vrem : real;

begin
  Form1.WS.Canvas.Pen.Color:=clRed;
  result:=10000;

  Form1.StatusBar1.Panels.Items[0].Text:='Ничего не выбрано';

  //Проверка выбора линии
  for count:=0 to k-1 do
  begin
     a:=sqrt(sqr(abs(Lines[count].x2-Lines[count].x1))+
                          sqr(abs(Lines[count].y2-Lines[count].y1)));

     b:=sqrt(sqr(abs(x-Lines[count].x1))+
                          sqr(abs(y-Lines[count].y1)));

     c:=sqrt(sqr(abs(x-Lines[count].x2))+
                          sqr(abs(y-Lines[count].y2)));

     p:=(a+b+c)/2;

     if (a<>0) and (b<>0) and (c<>0) then
           h:=Round(2*sqrt(p*abs(p-a)*abs(p-b)*abs(p-c))/a);

     if (a=0) or (b=0) or (c=0) then continue;

     alpha:=RadToDeg(ArcCos((Power(a,2)+Power(b,2)-Power(c,2))/(2*a*b)));
     beta:=RadToDeg(ArcCos((Power(a,2)+Power(c,2)-Power(b,2))/(2*a*c)));
     gamma:=RadToDeg(ArcCos((Power(b,2)+Power(c,2)-Power(a,2))/(2*b*c)));

     if (alpha>=90) or (beta>=90) or (gamma>=90) then
                   if gamma<90 then continue;

     if (h<5)then
      begin
          if (lines[count].x1<>lines[count].x2) then
            angle_vrem :=  ArcTan(abs(lines[count].y1-lines[count].y2)/abs(lines[count].x1-lines[count].x2))
          else
            angle_vrem:=DegToRad(90);
          angle_vrem := RadToDeg(angle_vrem);

          if ((lines[count].x2>lines[count].x1) and
             (lines[count].y2>lines[count].y1)) or
             ((lines[count].x1>lines[count].x2) and
             (lines[count].y1>lines[count].y2))
             then angle_vrem := -angle_vrem;

          angle_vrem := SimpleRoundTo(angle_vrem,-3);
          Form1.StatusBar1.Panels.Items[0].Text:='L = '+IntToStr(Round(a)) +
                                                 ' : Угол = '+ FloatToStr(angle_vrem);

          //////////////////////////////////////////////////////////////////////
          //Проверка выделения блока по примитиву Линия
          block_num := Prov_Sel_Block(count);
          if block_num<>-1 then  //Линия входит в блок
             begin
                 Reset_mode;
                 Select_block(block_num);
                 Form1.StatusBar1.Panels.Items[0].Text:='Блок №' + block_num.ToString;
             end
          //////////////////////////////////////////////////////////////////////
          else
             begin
                form1.WS.Canvas.Ellipse(Lines[count].x1-5+Screen_x,Lines[count].y1-5+screen_y,
                                 Lines[count].x1+5+Screen_x,Lines[count].y1+5+Screen_y);
                form1.WS.Canvas.Ellipse(Lines[count].x2-5+Screen_x,Lines[count].y2-5+Screen_y,
                                 Lines[count].x2+5+Screen_x,Lines[count].y2+5+Screen_y);
                result:=count;
                select_type:=1;
             end;
      end;
  end;

  //Проверка выбора круга
  for count:=0 to k_cir-1 do
  begin
      if (sqrt(sqr(x-Circles[count].x)+sqr(y-Circles[count].y))<Circles[count].r+5) and
         (sqrt(sqr(x-Circles[count].x)+sqr(y-Circles[count].y))>Circles[count].r-5) then
       begin
          //////////////////////////////////////////////////////////////////////
          //Проверка выделения блока по примитиву Круг
          block_num := Prov_Sel_Block(count);
          if block_num<>-1 then  //Круг входит в блок
             begin
                 Reset_mode;
                 Select_block(block_num);
                 Form1.StatusBar1.Panels.Items[0].Text:='Блок №' + block_num.ToString;
             end
          //////////////////////////////////////////////////////////////////////
          else
             begin
                  result:=count;
                  select_type:=2;

                  Form1.StatusBar1.Panels.Items[0].Text:='R = '+IntToStr(Circles[count].r);

                  Form1.WS.Canvas.Ellipse(Circles[count].x-Circles[count].r-5+Screen_x,
                                       Circles[count].y+Circles[count].r+5+Screen_y,
                                       Circles[count].x-Circles[count].r+5+Screen_x,
                                       Circles[count].y+Circles[count].r-5+Screen_y);
                  Form1.WS.Canvas.Ellipse(Circles[count].x-Circles[count].r-5+Screen_x,
                                       Circles[count].y-Circles[count].r+5+Screen_y,
                                       Circles[count].x-Circles[count].r+5+Screen_x,
                                       Circles[count].y-Circles[count].r-5+Screen_y);
                  Form1.WS.Canvas.Ellipse(Circles[count].x+Circles[count].r-5+Screen_x,
                                       Circles[count].y+Circles[count].r+5+Screen_y,
                                       Circles[count].x+Circles[count].r+5+Screen_x,
                                       Circles[count].y+Circles[count].r-5+Screen_y);
                  Form1.WS.Canvas.Ellipse(Circles[count].x+Circles[count].r-5+Screen_x,
                                       Circles[count].y-Circles[count].r+5+Screen_y,
                                       Circles[count].x+Circles[count].r+5+Screen_x,
                                       Circles[count].y-Circles[count].r-5+Screen_y);
                  Form1.WS.Canvas.Ellipse(Circles[count].x-3+Screen_x,Circles[count].y-3+Screen_y,
                                          Circles[count].x+3+Screen_x,Circles[count].y+3+Screen_y);
            end;
       end;
  end;

  //Проверка выбора дуги
  for count:=0 to k_arc-1 do
  begin
      if ((x>=Arcs[count].x1) and (x<=Arcs[count].x2) and
         (y>=Arcs[count].y1) and (y<=Arcs[count].y2)) or

         ((x>=Arcs[count].x2) and (x<=Arcs[count].x1) and
         (y>=Arcs[count].y1) and (y<=Arcs[count].y2)) or

         ((x>=Arcs[count].x1) and (x<=Arcs[count].x2) and
         (y>=Arcs[count].y2) and (y<=Arcs[count].y1)) or

         ((x>=Arcs[count].x2) and (x<=Arcs[count].x1) and
         (y>=Arcs[count].y2) and (y<=Arcs[count].y1))
         then
       begin
            //////////////////////////////////////////////////////////////////////
            //Проверка выделения блока по примитиву Дуга
            block_num := Prov_Sel_Block(count);
            if block_num<>-1 then  //Дуга входит в блок
             begin
                 Reset_mode;
                 Select_block(block_num);
                 Form1.StatusBar1.Panels.Items[0].Text:='Блок №' + block_num.ToString;
             end
          //////////////////////////////////////////////////////////////////////
            else
            begin
                result:=count;
                select_type:=6;

                Form1.StatusBar1.Panels.Items[0].Text:='Дуга';

                Form1.WS.Canvas.Ellipse(Arcs[count].x1-5+Screen_x,
                                        Arcs[count].y1-5+Screen_y,
                                        Arcs[count].x1+5+Screen_x,
                                        Arcs[count].y1+5+Screen_y);
                Form1.WS.Canvas.Ellipse(Arcs[count].x2-5+Screen_x,
                                        Arcs[count].y2-5+Screen_y,
                                        Arcs[count].x2+5+Screen_x,
                                        Arcs[count].y2+5+Screen_y);
            end;
       end;
  end;

   //Проверка выбора точки
  for count:=0 to k_points do
  begin
      if (sqrt(sqr(abs(x-points[count].x))+sqr(abs(y-points[count].y)))<5) then
       begin
            //////////////////////////////////////////////////////////////////////
            //Проверка выделения блока по примитиву Точка
            block_num := Prov_Sel_Block(count);
            if block_num<>-1 then  //Точка входит в блок
             begin
                 Reset_mode;
                 Select_block(block_num);
                 Form1.StatusBar1.Panels.Items[0].Text:='Блок №' + block_num.ToString;
             end
          //////////////////////////////////////////////////////////////////////
            else
            begin
                result:=count;
                select_type:=3;

                Form1.WS.Canvas.Ellipse(points[count].x-15+Screen_x,points[count].y+15+Screen_y,
                                        points[count].x+15+Screen_x,points[count].y-15+Screen_y);
                Form1.StatusBar1.Panels.Items[0].Text:='Точка';
            end;
       end;
  end;

   //Проверка выбора текста
  for count:=0 to k_nadp do
  begin
      if (nadp[count].x<x) and
             ((nadp[count].x+Length(nadp[count].s)*nadp[count].h)>x) and
             (nadp[count].y<y) and
             ((nadp[count].y+nadp[count].h)>y) then
       begin
            //////////////////////////////////////////////////////////////////////
            //Проверка выделения блока по примитиву Надпись
            block_num := Prov_Sel_Block(count);
            if block_num<>-1 then  //Надпись входит в блок
             begin
                 Reset_mode;
                 Select_block(block_num);
                 Form1.StatusBar1.Panels.Items[0].Text:='Блок №' + block_num.ToString;
             end
          //////////////////////////////////////////////////////////////////////
            else
            begin
                result:=count;
                select_type:=4;

                Form1.WS.Canvas.Font.Size:=nadp[count].h;
                Form1.WS.Canvas.Font.Color:= clRed;
                Form1.WS.Canvas.TextOut(nadp[count].x+Screen_x,nadp[count].y+Screen_y,
                                                             nadp[count].s);
                Form1.StatusBar1.Panels.Items[0].Text:='Текст';
            end;
       end;
  end;

  //Проверка выбора стрелки
  for count:=0 to k_arrow do
  begin
      if (sqrt(sqr(abs(x-arrow[count].x))+sqr(abs(y-arrow[count].y)))<5) then
       begin
            //////////////////////////////////////////////////////////////////////
            //Проверка выделения блока по примитиву Стрелка
            block_num := Prov_Sel_Block(count);
            if block_num<>-1 then  //Стрелка входит в блок
             begin
                 Reset_mode;
                 Select_block(block_num);
                 Form1.StatusBar1.Panels.Items[0].Text:='Блок №' + block_num.ToString;
             end
          //////////////////////////////////////////////////////////////////////
            else
            begin
                result:=count;
                select_type:=5;

                Form1.WS.Canvas.Ellipse(arrow[count].x-5,arrow[count].y+5,
                                        arrow[count].x+5,arrow[count].y-5);
                if arrow[count].t = 'left' then
                      Form1.WS.Canvas.Ellipse(arrow[count].x+30-5+Screen_x,arrow[count].y+5+Screen_y,
                                              arrow[count].x+30+5+Screen_x,arrow[count].y-5+Screen_y);
                if arrow[count].t = 'right' then
                      Form1.WS.Canvas.Ellipse(arrow[count].x-30-5+Screen_x,arrow[count].y+5+Screen_y,
                                              arrow[count].x-30+5+Screen_x,arrow[count].y-5+Screen_y);
                if arrow[count].t = 'up' then
                      Form1.WS.Canvas.Ellipse(arrow[count].x-5+Screen_x,arrow[count].y+30+5+Screen_y,
                                              arrow[count].x+5+Screen_x,arrow[count].y+30-5+Screen_y);
                if arrow[count].t = 'down' then
                      Form1.WS.Canvas.Ellipse(arrow[count].x-5+Screen_x,arrow[count].y-30+5+Screen_y,
                                              arrow[count].x+5+Screen_x,arrow[count].y-30-5+Screen_y);
                Form1.StatusBar1.Panels.Items[0].Text:='Стрелка';
            end;
       end;
  end;

  Form1.WS.Canvas.Pen.Color:=clBlack;
end;

procedure Rotate(xr,yr : integer); //Поворот фигуры (линии)

var
  Angle, angle_start:real;

  count : integer;
  rad : float;

begin
   if (Length(multy_selected_line)<>0) or
      (Length(multy_selected_circle)<>0) or
      (Length(multy_selected_arcs)<>0) or
      (Length(multy_selected_point)<>0) or
      (Length(multy_selected_text)<>0) or
      (Length(multy_selected_arrow)<>0) then
  begin
      save_prev(); //Сохранить предыдущее состояние

      try
         Angle:=Round(StrToInt(InputBox('Поворот фигуры','Введи угол против час.стр.','0')));
         Angle:=(Angle*System.Pi/180);
      except
         Angle:=0;
      end;

      //Поворот ОТРЕЗКОВ
      for count := 0 to High(multy_selected_line) do
      begin
          //1 точка
          rad := sqrt(sqr(lines[multy_selected_line[count]].y1-yr)+
                      sqr(lines[multy_selected_line[count]].x1-xr));

          if (lines[multy_selected_line[count]].x1<>xr) then
             angle_start := math.DegToRad(90)- ArcTan((lines[multy_selected_line[count]].y1-yr)/
                                (lines[multy_selected_line[count]].x1-xr))
          else
             angle_start := 0;

          if (xr<lines[multy_selected_line[count]].x1) then  //кон.точка справа от точки вращения
          begin
               lines[multy_selected_line[count]].x1 := xr + Round(rad * sin(angle + angle_start));
               lines[multy_selected_line[count]].y1 := yr + Round(rad * cos(angle + angle_start));
          end
          else                                              //кон.точка слева от точки вращения
          begin
               If (lines[multy_selected_line[count]].x1<>xr) then
                   angle_start := -math.DegToRad(90)-ArcTan((lines[multy_selected_line[count]].y1-yr)/
                                (lines[multy_selected_line[count]].x1-xr))
               else
                   angle_start := 0;
               lines[multy_selected_line[count]].x1 := xr - Round(rad * sin(math.DegToRad(180) + angle + angle_start));
               lines[multy_selected_line[count]].y1 := yr + Round(rad * cos(angle + angle_start));
          end;

          //2 точка
          rad := sqrt(sqr(lines[multy_selected_line[count]].y2-yr)+
                      sqr(lines[multy_selected_line[count]].x2-xr));

          if (lines[multy_selected_line[count]].x2<>xr) then
             angle_start := math.DegToRad(90)-ArcTan((lines[multy_selected_line[count]].y2-yr)/
                                (lines[multy_selected_line[count]].x2-xr))
          else
             angle_start := 0;

          if (xr<lines[multy_selected_line[count]].x2) then  //кон.точка справа от точки вращения
          begin
               lines[multy_selected_line[count]].x2 := xr + Round(rad * sin(angle + angle_start));
               lines[multy_selected_line[count]].y2 := yr + Round(rad * cos(angle + angle_start));
          end
          else                                               //кон.точка слева от точки вращения
          begin
               if (lines[multy_selected_line[count]].x2<>xr) then
                   angle_start := -math.DegToRad(90)-ArcTan((lines[multy_selected_line[count]].y2-yr)/
                                (lines[multy_selected_line[count]].x2-xr))
               else
                   angle_start := 0;
               lines[multy_selected_line[count]].x2 := xr - Round(rad * sin(math.DegToRad(180) + angle + angle_start));
               lines[multy_selected_line[count]].y2 := yr + Round(rad * cos(angle + angle_start));
          end;
      end;

      //Поворот ОКРУЖНОСТЕЙ
      for count := 0 to High(multy_selected_circle) do
      begin
          rad := sqrt(sqr(circles[multy_selected_circle[count]].y-yr)+
                      sqr(circles[multy_selected_circle[count]].x-xr));

          if (xr<circles[multy_selected_circle[count]].x) then  //кон.точка справа от точки вращения
          begin
              if (circles[multy_selected_circle[count]].x<>xr) then
                  angle_start := math.DegToRad(90)- ArcTan((circles[multy_selected_circle[count]].y-yr)/
                                    (circles[multy_selected_circle[count]].x-xr))
              else
                  angle_start := 0;
              circles[multy_selected_circle[count]].x := xr + Round(rad * sin(angle + angle_start));
              circles[multy_selected_circle[count]].y := yr + Round(rad * cos(angle + angle_start));
          end
          else                                                  //кон.точка слева от точки вращения
          begin
               if (circles[multy_selected_circle[count]].x<>xr) then
                  angle_start := -math.DegToRad(90)-ArcTan((circles[multy_selected_circle[count]].y-yr)/
                                (circles[multy_selected_circle[count]].x-xr))
               else
                  angle_start := 0;

               circles[multy_selected_circle[count]].x := xr - Round(rad * sin(math.DegToRad(180) + angle + angle_start));
               circles[multy_selected_circle[count]].y := yr + Round(rad * cos(angle + angle_start));
          end;
      end;

      //Поворот ТОЧЕК
      for count := 0 to High(multy_selected_point) do
      begin
          rad := sqrt(sqr(points[multy_selected_point[count]].y-yr)+
                      sqr(points[multy_selected_point[count]].x-xr));

          if (xr<points[multy_selected_point[count]].x) then  //кон.точка справа от точки вращения
          begin
              if (points[multy_selected_point[count]].x<>xr) then
                 angle_start := math.DegToRad(90)- ArcTan((points[multy_selected_point[count]].y-yr)/
                                    (points[multy_selected_point[count]].x-xr))
              else
                 angle_start := 0;
              points[multy_selected_point[count]].x := xr + Round(rad * sin(angle + angle_start));
              points[multy_selected_point[count]].y := yr + Round(rad * cos(angle + angle_start));
          end
          else                                                //кон.точка слева от точки вращения
          begin
               if (points[multy_selected_point[count]].x<>xr) then
                  angle_start := -math.DegToRad(90)-ArcTan((points[multy_selected_point[count]].y-yr)/
                                (points[multy_selected_point[count]].x-xr))
               else
                  angle_start := 0;
               points[multy_selected_point[count]].x := xr - Round(rad * sin(math.DegToRad(180) + angle + angle_start));
               points[multy_selected_point[count]].y := yr + Round(rad * cos(angle + angle_start));
          end;
      end;

      //Поворот ТЕКСТОВ
      for count := 0 to High(multy_selected_text) do
      begin
          rad := sqrt(sqr(nadp[multy_selected_text[count]].y-yr)+
                      sqr(nadp[multy_selected_text[count]].x-xr));

          if (xr<nadp[multy_selected_text[count]].x) then  //кон.точка справа от точки вращения
          begin
              if (nadp[multy_selected_text[count]].x<>xr) then
                 angle_start := math.DegToRad(90)- ArcTan((nadp[multy_selected_text[count]].y-yr)/
                                    (nadp[multy_selected_text[count]].x-xr))
              else
                 angle_start := 0;
              nadp[multy_selected_text[count]].x := xr + Round(rad * sin(angle + angle_start));
              nadp[multy_selected_text[count]].y := yr + Round(rad * cos(angle + angle_start));
          end
          else                                             //кон.точка слева от точки вращения
          begin
               if (nadp[multy_selected_text[count]].x<>xr) then
                  angle_start := -math.DegToRad(90)-ArcTan((nadp[multy_selected_text[count]].y-yr)/
                                (nadp[multy_selected_text[count]].x-xr))
               else
                  angle_start := 0;
               nadp[multy_selected_text[count]].x := xr - Round(rad * sin(math.DegToRad(180) + angle + angle_start));
               nadp[multy_selected_text[count]].y := yr + Round(rad * cos(angle + angle_start));
          end;
      end;

      //Поворот СТРЕЛОК
      for count := 0 to High(multy_selected_arrow) do
      begin
          rad := sqrt(sqr(arrow[multy_selected_arrow[count]].y-yr)+
                      sqr(arrow[multy_selected_arrow[count]].x-xr));

          if (xr<arrow[multy_selected_arrow[count]].x) then  //кон.точка справа от точки вращения
          begin
              angle_start := math.DegToRad(90)- ArcTan((arrow[multy_selected_arrow[count]].y-yr)/
                                    (arrow[multy_selected_arrow[count]].x-xr));
              arrow[multy_selected_arrow[count]].x := xr + Round(rad * sin(angle + angle_start));
              arrow[multy_selected_arrow[count]].y := yr + Round(rad * cos(angle + angle_start));
          end
          else                                               //кон.точка слева от точки вращения
          begin
               angle_start := -math.DegToRad(90)-ArcTan((arrow[multy_selected_arrow[count]].y-yr)/
                                (arrow[multy_selected_arrow[count]].x-xr));
               arrow[multy_selected_arrow[count]].x := xr - Round(rad * sin(math.DegToRad(180) + angle + angle_start));
               arrow[multy_selected_arrow[count]].y := yr + Round(rad * cos(angle + angle_start));
          end;
      end;

      clear;
  end;
end;

end.