unit View;

{
    Vector graphical editor.
    Version: 1.
    Written on FreePascal (https://freepascal.org/).
    Copyright (C) 2023  Warfalomay

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
}

{$mode ObjFPC}{$H+}

interface
uses Graphics;

//Перерисовать экран
procedure Clear();
//Подготовка изображения для сохранения в растр
procedure prepare_pic (pic : TPortableNetworkGraphic);
//Отрисовка нового места расположения объектов при копировании
//или перетаскивании
procedure DrawMoved(ShiftX,ShiftY : integer);

implementation
uses
  Unit1,Classes, SysUtils, dialogs;


procedure Clear();

var
  i:integer;
  vrem : TBitmap;

begin
  Vrem := TBitmap.Create();
  Vrem.Width := World_size;
  Vrem.Height := World_size;

  Vrem.Canvas.Refresh;

  Vrem.Canvas.Brush.Color:=clWhite;
  Vrem.Canvas.FillRect(Rect(0,0,Vrem.Width,Vrem.Height));
  Vrem.Canvas.Brush.Style:=bsClear;

  if (podl=true) and (show_back = true)  //Показывать подложку
                     then Vrem.Canvas.draw(0,0,back_png);

  for i:=0 to Arr_length-1 do
  begin
      with Lines[i] do
      begin
           if (stad) and (i<>k) then
           begin
               Vrem.Canvas.Pen.Style:=Lines[i].style;
               Vrem.Canvas.Pen.Width:=Lines[i].Width;
               Vrem.Canvas.Pen.Color:=Lines[i].col;
               Vrem.Canvas.Line(x1,y1,x2,y2);
           end;
           if not (stad) then
           begin
               Vrem.Canvas.Pen.Style:=Lines[i].style;
               Vrem.Canvas.Pen.Width:=Lines[i].Width;
               Vrem.Canvas.Pen.Color:=Lines[i].col;
               Vrem.Canvas.Line(x1,y1,x2,y2);
           end;
      end;

      with Circles[i] do
      begin
          if (Circles[i].x > 0) and
             (Circles[i].y > 0) then
          begin
              Vrem.Canvas.Pen.Style:=Circles[i].style;
              Vrem.Canvas.Pen.Width:=Circles[i].Width;
              Vrem.Canvas.Pen.Color:=Circles[i].col;
              Vrem.Canvas.Ellipse(Circles[i].x-Circles[i].r,
                                   Circles[i].y-Circles[i].r,
                                   Circles[i].x+Circles[i].r,
                                   Circles[i].y+Circles[i].r);

              Vrem.Canvas.Pen.Style:=psSolid;
              Vrem.Canvas.Pen.Width:=1;
              Vrem.Canvas.Pen.Color:=clGray;
          end;

          If not (drag_point_mode) then
          begin
              If P_show then
              begin
                  //Центр
                  Vrem.Canvas.Ellipse(Circles[i].x-2, Circles[i].y-2,
                                          Circles[i].x+2, Circles[i].y+2);
                  //Ординаты
                  Vrem.Canvas.Ellipse(Circles[i].x-Circles[i].r-2, Circles[i].y-2,
                                          Circles[i].x-Circles[i].r+2, Circles[i].y+2);
                  Vrem.Canvas.Ellipse(Circles[i].x+Circles[i].r-2, Circles[i].y-2,
                                          Circles[i].x+Circles[i].r+2, Circles[i].y+2);
                  Vrem.Canvas.Ellipse(Circles[i].x-2, Circles[i].y-Circles[i].r-2,
                                          Circles[i].x+2, Circles[i].y-Circles[i].r+2);
                  Vrem.Canvas.Ellipse(Circles[i].x-2, Circles[i].y+Circles[i].r-2,
                                          Circles[i].x+2, Circles[i].y+Circles[i].r+2);
              end;
          end;
      end;

      with arcs[i] do
      begin
          Vrem.Canvas.Pen.Style:=arcs[i].style;
          Vrem.Canvas.Pen.Width:=arcs[i].Width;
          Vrem.Canvas.Pen.Color:=arcs[i].col;
          Vrem.canvas.Arc(Arcs[i].l_x,Arcs[i].t_y,
                              Arcs[i].r_x,Arcs[i].b_y,
                              Arcs[i].x1,Arcs[i].y1,Arcs[i].x2,Arcs[i].y2);
      end;

      If P_Show then    //Показывать точки (вкл.по умолчанию)
      begin
          with points[i] do
          begin
               If (points[i].x<>0) and (points[i].y<>0) then
               begin
                   Vrem.Canvas.Pen.Color:=clGreen;
                   Vrem.Canvas.Pen.Width:=1;
                   Vrem.Canvas.Ellipse(points[i].x-5,
                                   points[i].y+5,points[i].x+5,points[i].y-5);
               end;
          end;
      end;

      with nadp[i] do
      begin
           Vrem.Canvas.Font.Size:=nadp[i].h;
           Vrem.Canvas.Font.Color:= nadp[i].col;
           Vrem.Canvas.Pen.Width:=1;
           Vrem.Canvas.TextOut(nadp[i].x,nadp[i].y,
                                   nadp[i].s);
      end;

      with arrow[i] do
      begin
           Vrem.Canvas.Pen.Color:=arrow[i].col;
           Vrem.Canvas.Pen.Width:=1;
           If arrow[i].t='left' then
           begin
               Vrem.Canvas.Line(x,y,x+30,y);
               Vrem.Canvas.Line(x,y,x+10,y+5);
               Vrem.Canvas.Line(x,y,x+10,y-5);
           end;
           If arrow[i].t='right' then
           begin
               Vrem.Canvas.Line(x,y,x-30,y);
               Vrem.Canvas.Line(x,y,x-10,y+5);
               Vrem.Canvas.Line(x,y,x-10,y-5);
           end;
           If arrow[i].t='up' then
           begin
               Vrem.Canvas.Line(x,y,x,y+30);
               Vrem.Canvas.Line(x,y,x+5,y+10);
               Vrem.Canvas.Line(x,y,x-5,y+10);
           end;
           If arrow[i].t='down' then
           begin
               Vrem.Canvas.Line(x,y,x,y-30);
               Vrem.Canvas.Line(x,y,x+5,y-10);
               Vrem.Canvas.Line(x,y,x-5,y-10);
           end;
      end;
  end;
  Form1.WS.Canvas.Draw(0+Screen_x,0+Screen_y,vrem);
  vrem.Free;
  //showmessage(Form1.WS.Left.ToString);

  with Form1 do
   imgTemp.Canvas.CopyRect(Rect(0,0,WS.Width, WS.Height), WS.Canvas, Rect(0,0,WS.Width, WS.Height));
end;

procedure prepare_pic (pic : TPortableNetworkGraphic);

var
  i : integer;

begin
    save_png.Canvas.Brush.Color:=clWhite;
    save_png.Canvas.FillRect(Rect(0,0,save_png.Width,save_png.Height));
    save_png.Canvas.Brush.Style:=bsClear;

    if (podl=true) and (show_back = true)  //Показывать подложку
                           then save_png.Canvas.draw(0,0,back_png);

    for i:=0 to Arr_length-1 do
        begin
          with Lines[i] do
          begin
               save_png.Canvas.Pen.Style:=Lines[i].style;
               save_png.Canvas.Pen.Width:=Lines[i].Width;
               save_png.Canvas.Pen.Color:=Lines[i].col;
               save_png.Canvas.Line(x1,y1,x2,y2);
          end;

          with Circles[i] do
          begin
              save_png.Canvas.Pen.Style:=Circles[i].style;
              save_png.Canvas.Pen.Width:=Circles[i].Width;
              save_png.Canvas.Pen.Color:=Circles[i].col;
              save_png.Canvas.Ellipse(Circles[i].x-Circles[i].r,
                                   Circles[i].y-Circles[i].r,
                                   Circles[i].x+Circles[i].r,
                                   Circles[i].y+Circles[i].r);
          end;

          with arcs[i] do
          begin
              save_png.Canvas.Pen.Style:=arcs[i].style;
              save_png.Canvas.Pen.Width:=arcs[i].Width;
              save_png.Canvas.Pen.Color:=arcs[i].col;
              save_png.canvas.Arc(Arcs[i].l_x,Arcs[i].t_y,
                                  Arcs[i].r_x,Arcs[i].b_y,
                                  Arcs[i].x1,Arcs[i].y1,Arcs[i].x2,Arcs[i].y2);
          end;

          If P_Show then    //Показывать точки (вкл.по умолчанию)
          begin
              with points[i] do
              begin
                   If (points[i].x<>0) and (points[i].y<>0) then
                   begin
                       save_png.Canvas.Pen.Color:=clGreen;
                       save_png.Canvas.Pen.Width:=1;
                       save_png.Canvas.Ellipse(points[i].x-5,
                                       points[i].y+5,points[i].x+5,points[i].y-5);
                   end;
              end;
          end;

          with nadp[i] do
          begin
               save_png.Canvas.Font.Size:=nadp[i].h;
               save_png.Canvas.Font.Color:= nadp[i].col;
               save_png.Canvas.Pen.Width:=1;
               save_png.Canvas.TextOut(nadp[i].x,nadp[i].y,
                                       nadp[i].s);
          end;


          with arrow[i] do
          begin
               save_png.Canvas.Pen.Color:=arrow[i].col;
               save_png.Canvas.Pen.Width:=1;
               If arrow[i].t='left' then
               begin
                   save_png.Canvas.Line(x,y,x+30,y);
                   save_png.Canvas.Line(x,y,x+10,y+5);
                   save_png.Canvas.Line(x,y,x+10,y-5);
               end;
               If arrow[i].t='right' then
               begin
                   save_png.Canvas.Line(x,y,x-30,y);
                   save_png.Canvas.Line(x,y,x-10,y+5);
                   save_png.Canvas.Line(x,y,x-10,y-5);
               end;
               If arrow[i].t='up' then
               begin
                   save_png.Canvas.Line(x,y,x,y+30);
                   save_png.Canvas.Line(x,y,x+5,y+10);
                   save_png.Canvas.Line(x,y,x-5,y+10);
               end;
               If arrow[i].t='down' then
               begin
                   save_png.Canvas.Line(x,y,x,y-30);
                   save_png.Canvas.Line(x,y,x+5,y-10);
                   save_png.Canvas.Line(x,y,x-5,y-10);
               end;

          end;
        end;
end;

procedure DrawMoved(ShiftX,ShiftY : integer);

var
  count : integer;

begin
    With Form1 do
    begin
        If selected = 10000 then
        begin
            for count := 0 to High(multy_selected_line) do
            begin
               WS.Canvas.Line(Lines[multy_selected_line[count]].x1 + ShiftX,
                              Lines[multy_selected_line[count]].y1 + ShiftY,
                              Lines[multy_selected_line[count]].x2 + ShiftX,
                              Lines[multy_selected_line[count]].y2 + ShiftY);
            end;
            for count := 0 to High(multy_selected_circle) do
            begin
               WS.Canvas.Ellipse(circles[multy_selected_circle[count]].x-
                                 circles[multy_selected_circle[count]].r + ShiftX,
                                 circles[multy_selected_circle[count]].y-
                                 circles[multy_selected_circle[count]].r + ShiftY,
                                 circles[multy_selected_circle[count]].x+
                                 circles[multy_selected_circle[count]].r + ShiftX,
                                 circles[multy_selected_circle[count]].y+
                                 circles[multy_selected_circle[count]].r + ShiftY)
            end;
            for count := 0 to High(multy_selected_arcs) do
            begin
               WS.Canvas.Arc(Arcs[count].l_x+ShiftX,Arcs[count].t_y+ShiftY,
                                  Arcs[count].r_x+ShiftX,Arcs[count].b_y+ShiftY,
                                  Arcs[count].x1+ShiftX,Arcs[count].y1+ShiftY,
                                  Arcs[count].x2+ShiftX,Arcs[count].y2+ShiftY);
            end;
            for count := 0 to High(multy_selected_point) do
            begin
               WS.Canvas.Ellipse(points[count].x-5+ShiftX,points[count].y+5+ShiftY,
                               points[count].x+5+ShiftX,points[count].y-5+ShiftY);
            end;
            for count := 0 to High(multy_selected_text) do
            begin
               WS.Canvas.Font.Size:=nadp[count].h;
               WS.Canvas.TextOut(nadp[count].x+ShiftX,nadp[count].y+ShiftY,
                                 nadp[count].s);
            end;
            for count := 0 to High(multy_selected_arrow) do
            begin
               If arrow[count].t='left' then
               begin
                   WS.Canvas.Line(arrow[count].x+ShiftX,arrow[count].y+ShiftY,
                                  arrow[count].x+30+ShiftX,arrow[count].y+ShiftY);
                   WS.Canvas.Line(arrow[count].x+ShiftX,arrow[count].y+ShiftY,
                                  arrow[count].x+10+ShiftX,arrow[count].y+5+ShiftY);
                   WS.Canvas.Line(arrow[count].x+ShiftX,arrow[count].y+ShiftY,
                                  arrow[count].x+10+ShiftX,arrow[count].y-5+ShiftY);
               end;
               If arrow[count].t='right' then
               begin
                   WS.Canvas.Line(arrow[count].x+ShiftX,arrow[count].y+ShiftY,
                                  arrow[count].x+ShiftX-30,arrow[count].y+ShiftY);
                   WS.Canvas.Line(arrow[count].x+ShiftX,arrow[count].y+ShiftY,
                                  arrow[count].x+ShiftX-10,arrow[count].y+5+ShiftY);
                   WS.Canvas.Line(arrow[count].x+ShiftX,arrow[count].y+ShiftY,
                                  arrow[count].x+ShiftX-10,arrow[count].y-5+ShiftY);
               end;
               If arrow[count].t='up' then
               begin
                   WS.Canvas.Line(arrow[count].x+ShiftX,arrow[count].y+ShiftY,
                                  arrow[count].x+ShiftX,arrow[count].y+30+ShiftY);
                   WS.Canvas.Line(arrow[count].x+ShiftX,arrow[count].y+ShiftY,
                                  arrow[count].x+5+ShiftX,arrow[count].y+10+ShiftY);
                   WS.Canvas.Line(arrow[count].x+ShiftX,arrow[count].y+ShiftY,
                                  arrow[count].x-5+ShiftX,arrow[count].y+10+ShiftY);
               end;
               If arrow[count].t='down' then
               begin
                   WS.Canvas.Line(arrow[count].x+ShiftX,arrow[count].y+ShiftY,
                                  arrow[count].x+ShiftX,arrow[count].y-30+ShiftY);
                   WS.Canvas.Line(arrow[count].x+ShiftX,arrow[count].y+ShiftY,
                                  arrow[count].x+5+ShiftX,arrow[count].y-10+ShiftY);
                   WS.Canvas.Line(arrow[count].x+ShiftX,arrow[count].y+delta_y,
                                  arrow[count].x-5+ShiftX,arrow[count].y-10+delta_y);
               end;
            end;
        end
        else
        begin
            Case select_type of
            1 : WS.Canvas.Line(Lines[selected].x1 + ShiftX,
                               Lines[selected].y1 + ShiftY,
                               Lines[selected].x2 + ShiftX,
                               Lines[selected].y2 + ShiftY);
            2 : WS.Canvas.Ellipse(circles[selected].x - circles[selected].r + ShiftX,
                                  circles[selected].y - circles[selected].r + ShiftY,
                                  circles[selected].x + circles[selected].r + ShiftX,
                                  circles[selected].y + circles[selected].r + ShiftY);
            3 : WS.Canvas.Ellipse(points[selected].x - 5 + ShiftX,
                                  points[selected].y - 5 + ShiftY,
                                  points[selected].x + 5 + ShiftX,
                                  points[selected].y + 5 + ShiftY);
            4 :
                begin
                    WS.Canvas.Font.Size:=nadp[selected].h;
                    WS.Canvas.TextOut(nadp[selected].x+ShiftX,nadp[selected].y+ShiftY,
                                      nadp[selected].s);
                end;
            5:
               begin
                   If arrow[selected].t='left' then
                   begin
                       WS.Canvas.Line(arrow[selected].x+ShiftX,arrow[selected].y+ShiftY,
                                      arrow[selected].x+30+ShiftX,arrow[selected].y+ShiftY);
                       WS.Canvas.Line(arrow[selected].x+ShiftX,arrow[selected].y+ShiftY,
                                      arrow[selected].x+10+ShiftX,arrow[selected].y+5+ShiftY);
                       WS.Canvas.Line(arrow[selected].x+ShiftX,arrow[selected].y+ShiftY,
                                      arrow[selected].x+10+ShiftX,arrow[selected].y-5+ShiftY);
                   end;
                   If arrow[selected].t='right' then
                   begin
                       WS.Canvas.Line(arrow[selected].x+ShiftX,arrow[selected].y+ShiftY,
                                      arrow[selected].x+ShiftX-30,arrow[selected].y+ShiftY);
                       WS.Canvas.Line(arrow[selected].x+ShiftX,arrow[selected].y+ShiftY,
                                      arrow[selected].x+ShiftX-10,arrow[selected].y+5+ShiftY);
                       WS.Canvas.Line(arrow[selected].x+ShiftX,arrow[selected].y+ShiftY,
                                      arrow[selected].x+ShiftX-10,arrow[selected].y-5+ShiftY);
                   end;
                   If arrow[selected].t='up' then
                   begin
                       WS.Canvas.Line(arrow[selected].x+ShiftX,arrow[selected].y+ShiftY,
                                      arrow[selected].x+ShiftX,arrow[selected].y+30+ShiftY);
                       WS.Canvas.Line(arrow[selected].x+ShiftX,arrow[selected].y+ShiftY,
                                      arrow[selected].x+5+ShiftX,arrow[selected].y+10+ShiftY);
                       WS.Canvas.Line(arrow[selected].x+ShiftX,arrow[selected].y+ShiftY,
                                      arrow[selected].x-5+ShiftX,arrow[selected].y+10+ShiftY);
                   end;
                   If arrow[selected].t='down' then
                   begin
                       WS.Canvas.Line(arrow[selected].x+ShiftX,arrow[selected].y+ShiftY,
                                      arrow[selected].x+ShiftX,arrow[selected].y-30+ShiftY);
                       WS.Canvas.Line(arrow[selected].x+ShiftX,arrow[selected].y+ShiftY,
                                      arrow[selected].x+5+ShiftX,arrow[selected].y-10+ShiftY);
                       WS.Canvas.Line(arrow[selected].x+ShiftX,arrow[selected].y+delta_y,
                                      arrow[selected].x-5+ShiftX,arrow[selected].y-10+delta_y);
                   end;
               end;
            6: WS.Canvas.Arc(Arcs[selected].l_x+ShiftX,Arcs[selected].t_y+ShiftY,
                             Arcs[selected].r_x+ShiftX,Arcs[selected].b_y+ShiftY,
                             Arcs[selected].x1+ShiftX,Arcs[selected].y1+ShiftY,
                             Arcs[selected].x2+ShiftX,Arcs[selected].y2+ShiftY);
            end;
        end;
    end;
end;

end.